#include <stdint.h>
#include <stdio.h>
#include <tgmath.h>

#include "devices.h"
#include "instruments.h"
#include "drv_instrument.h"
#include "drv_dac.h"

#define PI 3.14159
#define DAC_CHAN_X 0
#define DAC_CHAN_Y 1

dac_t *dac;
instrument_t *ctlpanel;

void init() {
    dac = dac_open(DAC_1);
    ctlpanel = instrument_open(CTLPANEL);
}

void main() {
    init();
    printf("NOS031003 started.\n");
    uint32_t xv,yv,lastx,lasty;
    bool upd;
    lastx = 999;
    lasty = 999;
    uint8_t gen1x,gen1y,gen2x,gen2y,gen3x,gen3y,mul1x,mul1y,mul2x,mul2y,mul3x,mul3y;
    uint32_t tim, ltim;
    uint8_t sel_saw,sel_sine,sel_man;
    tim = 0;
    ltim = 0;
    for (;;) {
        tim++;
        gen3x = instrument_get_value(ctlpanel,CTLPANEL_XIN);
        gen3y = instrument_get_value(ctlpanel,CTLPANEL_YIN);
        sel_saw = instrument_get_value(ctlpanel,CTLPANEL_SEL_SAW);
        sel_sine = instrument_get_value(ctlpanel,CTLPANEL_SEL_SINE);
        sel_man = instrument_get_value(ctlpanel,CTLPANEL_SEL_MAN);
        mul1x = mul1y = (sel_saw!=0)?1:0;
        mul2x = mul2y = (sel_sine!=0)?1:0;
        mul3x = mul3y = (sel_man!=0)?1:0;
        /* if (tim<1000) continue; */
        gen1x = (gen1x+1) % 256;
        gen1y = (gen1y+3) % 256;
        gen2x = 128*(1+(sin((tim%100)*2*PI/100)));
        gen2y = 128*(1+(cos((tim%100)*2*PI/100)));
        xv = (mul1x*gen1x) + (mul2x*gen2x) + (mul3x*gen3x);
        yv = (mul1y*gen1y) + (mul2y*gen2y) + (mul3y*gen3y);
        upd = false;
        if ((xv!=lastx) || (yv!=lasty)) {
            upd = true;
            lastx = xv;
            lasty = yv;
        }
        if (upd) {
           /* printf("X=%d Y=%d\n",xv,yv); */
           ltim++;
           if (ltim>2) {
              instrument_set_value(ctlpanel,CTLPANEL_XOUT,xv);
              instrument_set_value(ctlpanel,CTLPANEL_YOUT,yv);
              ltim = 0;
           }
           dac_write(dac,DAC_CHAN_X,xv,false);
           dac_write(dac,DAC_CHAN_Y,yv,true);
        }
    }
}

